@@ -118,6 +118,51 @@ void KFileItemTest::testPermissionsString()
     QCOMPARE(unkwnownfileItem.getStatusBarInfo(), CaseInsensitiveStringCompareHelper(QStringLiteral("unkwnown_file (Unknown)")));
 }
 
+// https://bugs.kde.org/475422
+void KFileItemTest::testRelativeSymlinkGetStatusBarInfo()
+{
+#ifdef Q_OS_UNIX
+    QTemporaryDir tempDir;
+
+    // relative symlink to a file
+    {
+        KIO::UDSEntry entry;
+        entry.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("afile.relative"));
+        entry.fastInsert(KIO::UDSEntry::UDS_LINK_DEST, QStringLiteral("afile"));
+
+        KFileItem symlinkItem(entry, QUrl::fromLocalFile(tempDir.path()), true, true);
+        QCOMPARE(symlinkItem.getStatusBarInfo(),
+                 CaseInsensitiveStringCompareHelper(QStringLiteral("afile.relative (Unknown, Link to %1/afile)").arg(tempDir.path())));
+    }
+
+    // relative symlink to a file, name has spaces
+    {
+        KIO::UDSEntry entry;
+        entry.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("a file with spaces.relative"));
+        entry.fastInsert(KIO::UDSEntry::UDS_LINK_DEST, QStringLiteral("a file with spaces"));
+
+        KFileItem symlinkItem(entry, QUrl::fromLocalFile(tempDir.path()), true, true);
+        QCOMPARE(
+            symlinkItem.getStatusBarInfo(),
+            CaseInsensitiveStringCompareHelper(QStringLiteral("a file with spaces.relative (Unknown, Link to %1/a file with spaces)").arg(tempDir.path())));
+    }
+
+    // relative symlink in remote
+    {
+        KIO::UDSEntry entry;
+        entry.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("afile.relative"));
+        entry.fastInsert(KIO::UDSEntry::UDS_LINK_DEST, QStringLiteral("afile"));
+
+        QString remotePath = QStringLiteral("fish://192.168.1.1/tmp");
+
+        KFileItem symlinkItem(entry, QUrl(remotePath), true, true);
+        QCOMPARE(symlinkItem.getStatusBarInfo(),
+                 CaseInsensitiveStringCompareHelper(QStringLiteral("afile.relative (Unknown, Link to %1/afile)").arg(remotePath)));
+    }
+
+#endif
+}
+
 void KFileItemTest::testNull()
 {
     KFileItem null;
@@ -212,7 +257,7 @@ void KFileItemTest::testMimeTypeCtor()
     QVERIFY(!fileItem.isMimeTypeKnown());
 }
 
-void KFileItemTest::testBasic()
+void KFileItemTest::testBasicFile()
 {
     QTemporaryFile file;
     QVERIFY(file.open());
@@ -232,11 +277,39 @@ void KFileItemTest::testBasic()
     QVERIFY(fileItem.isReadable());
     QVERIFY(fileItem.isWritable());
     QVERIFY(fileItem.isFile());
+    QVERIFY(fileItem.isRegularFile());
     QVERIFY(!fileItem.isDir());
     QVERIFY(!fileItem.isDesktopFile());
+    QCOMPARE(fileItem.mimetype(), "text/plain");
+    // StatMimeType was not requested
+    QVERIFY(!fileItem.entry().contains(KIO::UDSEntry::UDS_MIME_TYPE));
 #ifndef Q_OS_WIN
     QCOMPARE(fileItem.user(), KUser().loginName());
 #endif
+    // TODO test date fields
+}
+
+void KFileItemTest::testBasicDirectory()
+{
+    QTemporaryDir dir;
+    QUrl dirUrl = QUrl::fromLocalFile(dir.path());
+    KFileItem dirItem(dirUrl, QString(), KFileItem::Unknown);
+    QCOMPARE(dirItem.text(), dirUrl.fileName());
+    QVERIFY(dirItem.isLocalFile());
+    QCOMPARE(dirItem.localPath(), dirUrl.toLocalFile());
+    QCOMPARE(dirItem.size(), KIO::filesize_t(40));
+    QVERIFY(dirItem.linkDest().isEmpty());
+    QVERIFY(!dirItem.isHidden());
+    QVERIFY(dirItem.isReadable());
+    QVERIFY(dirItem.isWritable());
+    QVERIFY(!dirItem.isFile());
+    QVERIFY(!dirItem.isRegularFile());
+    QVERIFY(dirItem.isDir());
+    QCOMPARE(dirItem.mimetype(), "inode/directory");
+    // StatMimeType was not requested
+    QVERIFY(!dirItem.entry().contains(KIO::UDSEntry::UDS_MIME_TYPE));
+    QVERIFY(!dirItem.isDesktopFile());
+    // TODO test date fields
 }
 
 void KFileItemTest::testRootDirectory()
