@@ -179,18 +179,19 @@ int KRunMX2::expandEscapedMacro(const QString &str, int pos, QStringList &ret)
 
 QStringList KIO::DesktopExecParser::supportedProtocols(const KService &service)
 {
-    QStringList supportedProtocols = service.property(QStringLiteral("X-KDE-Protocols")).toStringList();
+    QStringList supportedProtocols = service.supportedProtocols();
+
     KRunMX1 mx1(service);
     QString exec = service.exec();
     if (mx1.expandMacrosShellQuote(exec) && !mx1.hasUrls) {
         if (!supportedProtocols.isEmpty()) {
-            qCWarning(KIO_CORE) << service.entryPath() << "contains a X-KDE-Protocols line but doesn't use %u or %U in its Exec line! This is inconsistent.";
+            qCWarning(KIO_CORE) << service.entryPath() << "contains supported protocols but doesn't use %u or %U in its Exec line! This is inconsistent.";
         }
         return QStringList();
     } else {
         if (supportedProtocols.isEmpty()) {
             // compat mode: assume KIO if not set and it's a KDE app (or a KDE service)
-            const QStringList categories = service.property(QStringLiteral("Categories")).toStringList();
+            const QStringList categories = service.property<QStringList>(QStringLiteral("Categories"));
             if (categories.contains(QLatin1String("KDE")) || !service.isApplication() || service.entryPath().isEmpty() /*temp service*/) {
                 supportedProtocols.append(QStringLiteral("KIO"));
             } else { // if no KDE app, be a bit over-generic
@@ -201,27 +202,18 @@ QStringList KIO::DesktopExecParser::supportedProtocols(const KService &service)
         }
     }
 
-    // add x-scheme-handler/<protocol>
-    const auto servicesTypes = service.serviceTypes();
-    for (const auto &mimeType : servicesTypes) {
-        if (mimeType.startsWith(QLatin1String("x-scheme-handler/"))) {
-            supportedProtocols << mimeType.mid(17);
-        }
-    }
-
     // qCDebug(KIO_CORE) << "supportedProtocols:" << supportedProtocols;
     return supportedProtocols;
 }
 
 bool KIO::DesktopExecParser::isProtocolInSupportedList(const QUrl &url, const QStringList &supportedProtocols)
 {
-    if (supportedProtocols.contains(QLatin1String("KIO"))) {
-        return true;
-    }
-    return url.isLocalFile() || supportedProtocols.contains(url.scheme().toLower());
+    return url.isLocalFile() //
+        || supportedProtocols.contains(QLatin1String("KIO")) //
+        || supportedProtocols.contains(url.scheme(), Qt::CaseInsensitive);
 }
 
-// We have up to two sources of data, for protocols not handled by kioslaves (so called "helper") :
+// We have up to two sources of data, for protocols not handled by KIO workers (so called "helper") :
 // 1) the exec line of the .protocol file, if there's one
 // 2) the application associated with x-scheme-handler/<protocol> if there's one
 bool KIO::DesktopExecParser::hasSchemeHandler(const QUrl &url) // KF6 TODO move to OpenUrlJob
@@ -246,6 +238,8 @@ public:
     {
     }
 
+    bool isUrlSupported(const QUrl &url, const QStringList &supportedProtocols);
+
     const KService &service;
     QList<QUrl> urls;
     bool tempFiles;
@@ -309,6 +303,24 @@ static QString findNonExecutableProgram(const QString &executable)
     return QString();
 }
 
+bool KIO::DesktopExecParserPrivate::isUrlSupported(const QUrl &url, const QStringList &protocols)
+{
+    if (KIO::DesktopExecParser::isProtocolInSupportedList(url, protocols)) {
+        return true;
+    }
+
+    // supportedProtocols() only checks whether the .desktop file has MimeType=x-scheme-handler/xxx
+    // We also want to check whether the app has been set as default/associated in mimeapps.list
+    const auto handlers = KApplicationTrader::queryByMimeType(QLatin1String("x-scheme-handler/") + url.scheme());
+    for (const KService::Ptr &handler : handlers) {
+        if (handler->desktopEntryName() == service.desktopEntryName()) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 QStringList KIO::DesktopExecParser::resultingArguments() const
 {
     QString exec = d->service.exec();
@@ -365,7 +377,7 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
     // FIXME: the current way of invoking kioexec disables term and su use
 
     // Check if we need "tempexec" (kioexec in fact)
-    appHasTempFileOption = d->tempFiles && d->service.property(QStringLiteral("X-KDE-HasTempFileOption")).toBool();
+    appHasTempFileOption = d->tempFiles && d->service.property<bool>(QStringLiteral("X-KDE-HasTempFileOption"));
     if (d->tempFiles && !appHasTempFileOption && d->urls.size()) {
         result << kioexecPath() << QStringLiteral("--tempfiles") << exec;
         if (!d->suggestedFileName.isEmpty()) {
@@ -378,7 +390,7 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
 
     // Return true for non-KIO desktop files with explicit X-KDE-Protocols list, like vlc, for the special case below
     auto isNonKIO = [this]() {
-        const QStringList protocols = d->service.property(QStringLiteral("X-KDE-Protocols")).toStringList();
+        const QStringList protocols = d->service.property<QStringList>(QStringLiteral("X-KDE-Protocols"));
         return !protocols.isEmpty() && !protocols.contains(QLatin1String("KIO"));
     };
 
@@ -390,26 +402,18 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
         QDBusPendingReply<QString> reply;
         int urlIndex;
     };
-    QVector<MountRequest> requests;
+    QList<MountRequest> requests;
     requests.reserve(d->urls.count());
+
     const QStringList appSupportedProtocols = supportedProtocols(d->service);
     for (int i = 0; i < d->urls.count(); ++i) {
         const QUrl url = d->urls.at(i);
-        const bool supported = mx1.hasUrls ? isProtocolInSupportedList(url, appSupportedProtocols) : url.isLocalFile();
+        const bool supported = mx1.hasUrls ? d->isUrlSupported(url, appSupportedProtocols) : url.isLocalFile();
         if (!supported) {
-            // if FUSE fails, we'll have to fallback to kioexec
+            // If FUSE fails, and there is no scheme handler, we'll have to fallback to kioexec
             useKioexec = true;
         }
 
-        // supportedProtocols() only checks whether the .desktop file has MimeType=x-scheme-handler/xxx
-        // We also want to check whether the app has been set as default/associated in mimeapps.list
-        const auto handlers = KApplicationTrader::queryByMimeType(QLatin1String("x-scheme-handler/") + url.scheme());
-        for (const KService::Ptr &handler : handlers) {
-            if (handler->desktopEntryName() == d->service.desktopEntryName()) {
-                useKioexec = false;
-            }
-        }
-
         // NOTE: Some non-KIO apps may support the URLs (e.g. VLC supports smb://)
         // but will not have the password if they are not in the URL itself.
         // Hence convert URL to KIOFuse equivalent in case there is a password.
@@ -483,17 +487,23 @@ QStringList KIO::DesktopExecParser::resultingArguments() const
     */
 
     if (d->service.terminal()) {
-        KConfigGroup cg(KSharedConfig::openConfig(), "General");
+        KConfigGroup cg(KSharedConfig::openConfig(), QStringLiteral("General"));
         QString terminal = cg.readPathEntry("TerminalApplication", QStringLiteral("konsole"));
-        const bool isKonsole = (terminal == QLatin1String("konsole"));
 
-        QString terminalPath = QStandardPaths::findExecutable(terminal);
+        const bool isKonsole = (terminal == QLatin1String("konsole"));
+        QStringList terminalParts = KShell::splitArgs(terminal);
+        QString terminalPath;
+        if (!terminalParts.isEmpty()) {
+            terminalPath = QStandardPaths::findExecutable(terminalParts.at(0));
+        }
+
         if (terminalPath.isEmpty()) {
             d->m_errorString = i18n("Terminal %1 not found while trying to run %2", terminal, d->service.entryPath());
             qCWarning(KIO_CORE) << "Terminal" << terminal << "not found, service" << d->service.name();
             return QStringList();
         }
-        terminal = terminalPath;
+        terminalParts[0] = terminalPath;
+        terminal = KShell::joinArgs(terminalParts);
         if (isKonsole) {
             if (!d->service.workingDirectory().isEmpty()) {
                 terminal += QLatin1String(" --workdir ") + KShell::quoteArg(d->service.workingDirectory());
