@@ -10,14 +10,18 @@
 
 #include "kfileitem.h"
 
+#ifndef KIO_ANDROID_STUB
+#include "kdirnotify.h"
+#endif
+
 #include <QCache>
 #include <QCoreApplication>
 #include <QFileInfo>
 #include <QHash>
+#include <QList>
 #include <QMap>
 #include <QTimer>
 #include <QUrl>
-#include <QVector>
 
 #include <KDirWatch>
 #include <kio/global.h>
@@ -46,6 +50,14 @@ public:
     void slotProcessedSize(KJob *, qulonglong);
     void slotSpeed(KJob *, unsigned long);
 
+    /*
+     * Called by the public matchesMimeFilter() to do the
+     * actual filtering. Those methods may be reimplemented to customize
+     * filtering.
+     * @param mimeType the MIME type to filter
+     * @param filters the list of MIME types to filter
+     */
+    bool doMimeFilter(const QString &mimeType, const QStringList &filters) const;
     bool doMimeExcludeFilter(const QString &mimeExclude, const QStringList &filters) const;
     void connectJob(KIO::ListJob *);
     void jobDone(KIO::ListJob *);
@@ -56,6 +68,34 @@ public:
     void emitItems();
     void emitItemsDeleted(const KFileItemList &items);
 
+    /*
+     * Called for every new item before emitting newItems().
+     * You may reimplement this method in a subclass to implement your own
+     * filtering.
+     * The default implementation filters out ".." and everything not matching
+     * the name filter(s)
+     * @return @c true if the item is "ok".
+     *         @c false if the item shall not be shown in a view, e.g.
+     * files not matching a pattern *.cpp ( KFileItem::isHidden())
+     * @see matchesFilter
+     * @see setNameFilter
+     */
+    bool matchesFilter(const KFileItem &) const;
+
+    /*
+     * Called for every new item before emitting newItems().
+     * You may reimplement this method in a subclass to implement your own
+     * filtering.
+     * The default implementation filters out everything not matching
+     * the mime filter(s)
+     * @return @c true if the item is "ok".
+     *         @c false if the item shall not be shown in a view, e.g.
+     * files not matching the mime filter
+     * @see matchesMimeFilter
+     * @see setMimeFilter
+     */
+    bool matchesMimeFilter(const KFileItem &) const;
+
     /**
      * Redirect this dirlister from oldUrl to newUrl.
      * @param keepItems if true, keep the fileitems (e.g. when renaming an existing dir);
@@ -126,7 +166,7 @@ public:
         }
         bool isShowingDotFiles;
         bool dirOnlyMode;
-        QVector<QRegularExpression> lstFilters;
+        QList<QRegularExpression> lstFilters;
         QStringList mimeFilter;
         QStringList mimeExcludeFilter;
     };
@@ -153,8 +193,8 @@ class KCoreDirListerCache : public QObject
 {
     Q_OBJECT
 public:
-    KCoreDirListerCache(); // only called by K_GLOBAL_STATIC
-    ~KCoreDirListerCache();
+    KCoreDirListerCache(); // only called by QThreadStorage internally
+    ~KCoreDirListerCache() override;
 
     void updateDirectory(const QUrl &dir);
 
@@ -289,6 +329,17 @@ private:
         }
     }
 
+    void remove(const QUrl &oldUrl)
+    {
+        const QUrl parentDir = oldUrl.adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash);
+        DirItem *dirItem = dirItemForUrl(parentDir);
+        if (dirItem) {
+            auto it = std::lower_bound(dirItem->lstItems.begin(), dirItem->lstItems.end(), oldUrl);
+            Q_ASSERT(it != dirItem->lstItems.end());
+            dirItem->lstItems.erase(it);
+        }
+    }
+
     /**
      * When KDirWatch tells us that something changed in "dir", we need to
      * also notify the dirlisters that are listing a symlink to "dir" (#213799)
@@ -355,11 +406,13 @@ private:
         {
             // Note that "entering" means "start watching", and "leaving" means "stop watching"
             // (i.e. it's not when the user leaves the directory, it's when the directory is removed from the cache)
+#ifndef KIO_ANDROID_STUB
             if (entering) {
                 org::kde::KDirNotify::emitEnteredDirectory(url);
             } else {
                 org::kde::KDirNotify::emitLeftDirectory(url);
             }
+#endif
         }
 
         void redirect(const QUrl &newUrl)
@@ -415,6 +468,20 @@ private:
             lstItems.insert(it, item);
         }
 
+        // Insert the already sorted items in the sorted list
+        void insertSortedItems(const KFileItemList &items)
+        {
+            if (items.isEmpty()) {
+                return;
+            }
+            lstItems.reserve(lstItems.size() + items.size());
+            auto it = lstItems.begin();
+            for (const auto &item : items) {
+                it = std::lower_bound(it, lstItems.end(), item.url());
+                it = lstItems.insert(it, item);
+            }
+        }
+
         // number of KCoreDirListers using autoUpdate for this dir
         short autoUpdates;
 
@@ -502,7 +569,7 @@ public:
 
     /*reimp*/ void start() override
     {
-        QMetaObject::invokeMethod(this, "done", Qt::QueuedConnection);
+        QMetaObject::invokeMethod(this, &KCoreDirListerPrivate::CachedItemsJob::done, Qt::QueuedConnection);
     }
 
     // For updateDirectory() to cancel m_emitCompleted;
