@@ -10,7 +10,7 @@
 #include "job_p.h"
 #include "kprotocolinfo.h"
 #include "scheduler.h"
-#include "slave.h"
+#include "worker_p.h"
 #include <QDebug>
 #include <QTimer>
 #include <kdirnotify.h>
@@ -42,7 +42,7 @@ bool SimpleJob::doKill()
     Q_D(SimpleJob);
     if ((d->m_extraFlags & JobPrivate::EF_KillCalled) == 0) {
         d->m_extraFlags |= JobPrivate::EF_KillCalled;
-        Scheduler::cancelJob(this); // deletes the slave if not 0
+        Scheduler::cancelJob(this); // deletes the worker if not 0
     } else {
         qCWarning(KIO_CORE) << this << "killed twice, this is overkill";
     }
@@ -52,8 +52,8 @@ bool SimpleJob::doKill()
 bool SimpleJob::doSuspend()
 {
     Q_D(SimpleJob);
-    if (d->m_slave) {
-        d->m_slave->suspend();
+    if (d->m_worker) {
+        d->m_worker->suspend();
     }
     return Job::doSuspend();
 }
@@ -61,8 +61,8 @@ bool SimpleJob::doSuspend()
 bool SimpleJob::doResume()
 {
     Q_D(SimpleJob);
-    if (d->m_slave) {
-        d->m_slave->resume();
+    if (d->m_worker) {
+        d->m_worker->resume();
     }
     return Job::doResume();
 }
@@ -75,18 +75,18 @@ const QUrl &SimpleJob::url() const
 void SimpleJob::putOnHold()
 {
     Q_D(SimpleJob);
-    Q_ASSERT(d->m_slave);
-    if (d->m_slave) {
-        Scheduler::putSlaveOnHold(this, d->m_url);
+    Q_ASSERT(d->m_worker);
+    if (d->m_worker) {
+        Scheduler::putWorkerOnHold(this, d->m_url);
     }
-    // we should now be disassociated from the slave
-    Q_ASSERT(!d->m_slave);
+    // we should now be disassociated from the worker
+    Q_ASSERT(!d->m_worker);
     kill(Quietly);
 }
 
 void SimpleJob::removeOnHold()
 {
-    Scheduler::removeSlaveOnHold();
+    Scheduler::removeWorkerOnHold();
 }
 
 bool SimpleJob::isRedirectionHandlingEnabled() const
@@ -110,44 +110,44 @@ SimpleJob::~SimpleJob()
     }
 }
 
-void SimpleJobPrivate::start(Slave *slave)
+void SimpleJobPrivate::start(Worker *worker)
 {
     Q_Q(SimpleJob);
-    m_slave = slave;
+    m_worker = worker;
 
-    // Slave::setJob can send us SSL metadata if there is a persistent connection
-    QObject::connect(slave, &Slave::metaData, q, &SimpleJob::slotMetaData);
+    // Worker::setJob can send us SSL metadata if there is a persistent connection
+    QObject::connect(worker, &Worker::metaData, q, &SimpleJob::slotMetaData);
 
-    slave->setJob(q);
+    worker->setJob(q);
 
-    QObject::connect(slave, &Slave::error, q, &SimpleJob::slotError);
+    QObject::connect(worker, &Worker::error, q, &SimpleJob::slotError);
 
-    QObject::connect(slave, &Slave::warning, q, &SimpleJob::slotWarning);
+    QObject::connect(worker, &Worker::warning, q, &SimpleJob::slotWarning);
 
-    QObject::connect(slave, &Slave::finished, q, &SimpleJob::slotFinished);
+    QObject::connect(worker, &Worker::finished, q, &SimpleJob::slotFinished);
 
-    QObject::connect(slave, &Slave::infoMessage, q, [this](const QString &message) {
-        _k_slotSlaveInfoMessage(message);
+    QObject::connect(worker, &Worker::infoMessage, q, [this](const QString &message) {
+        _k_slotWorkerInfoMessage(message);
     });
 
-    QObject::connect(slave, &Slave::connected, q, [this]() {
+    QObject::connect(worker, &Worker::connected, q, [this]() {
         slotConnected();
     });
 
-    QObject::connect(slave, &Slave::privilegeOperationRequested, q, [this]() {
+    QObject::connect(worker, &Worker::privilegeOperationRequested, q, [this]() {
         slotPrivilegeOperationRequested();
     });
 
     if ((m_extraFlags & EF_TransferJobDataSent) == 0) { // this is a "get" job
-        QObject::connect(slave, &Slave::totalSize, q, [this](KIO::filesize_t size) {
+        QObject::connect(worker, &Worker::totalSize, q, [this](KIO::filesize_t size) {
             slotTotalSize(size);
         });
 
-        QObject::connect(slave, &Slave::processedSize, q, [this](KIO::filesize_t size) {
+        QObject::connect(worker, &Worker::processedSize, q, [this](KIO::filesize_t size) {
             slotProcessedSize(size);
         });
 
-        QObject::connect(slave, &Slave::speed, q, [this](ulong speed) {
+        QObject::connect(worker, &Worker::speed, q, [this](ulong speed) {
             slotSpeed(speed);
         });
     }
@@ -168,40 +168,35 @@ void SimpleJobPrivate::start(Slave *slave)
 
     if (!m_outgoingMetaData.isEmpty()) {
         KIO_ARGS << m_outgoingMetaData;
-        slave->send(CMD_META_DATA, packedArgs);
+        worker->send(CMD_META_DATA, packedArgs);
     }
 
-    if (!m_subUrl.isEmpty()) { // TODO KF6 remove
-        KIO_ARGS << m_subUrl;
-        slave->send(CMD_SUBURL, packedArgs);
-    }
-
-    slave->send(m_command, m_packedArgs);
+    worker->send(m_command, m_packedArgs);
     if (q->isSuspended()) {
-        slave->suspend();
+        worker->suspend();
     }
 }
 
-void SimpleJobPrivate::slaveDone()
+void SimpleJobPrivate::workerDone()
 {
     Q_Q(SimpleJob);
-    if (m_slave) {
+    if (m_worker) {
         if (m_command == CMD_OPEN) {
-            m_slave->send(CMD_CLOSE);
+            m_worker->send(CMD_CLOSE);
         }
-        q->disconnect(m_slave); // Remove all signals between slave and job
+        q->disconnect(m_worker); // Remove all signals between worker and job
     }
     // only finish a job once; Scheduler::jobFinished() resets schedSerial to zero.
     if (m_schedSerial) {
-        Scheduler::jobFinished(q, m_slave);
+        Scheduler::jobFinished(q, m_worker);
     }
 }
 
 void SimpleJob::slotFinished()
 {
     Q_D(SimpleJob);
-    // Return slave to the scheduler
-    d->slaveDone();
+    // Return worker to the scheduler
+    d->workerDone();
 
     if (!hasSubjobs()) {
         if (!error() && (d->m_command == CMD_MKDIR || d->m_command == CMD_RENAME)) {
@@ -215,8 +210,9 @@ void SimpleJob::slotFinished()
                 QUrl dst;
                 QDataStream str(d->m_packedArgs);
                 str >> src >> dst;
-                if (src.adjusted(QUrl::RemoveFilename) == dst.adjusted(QUrl::RemoveFilename) // For the user, moving isn't renaming. Only renaming is.
-                    && !KProtocolInfo::slaveHandlesNotify(dst.scheme()).contains(QLatin1String("Rename"))) {
+                if (src.adjusted(QUrl::RemoveFilename) == dst.adjusted(QUrl::RemoveFilename) // For the user, moving isn't
+                                                                                             // renaming. Only renaming is.
+                ) {
 #ifndef KIO_ANDROID_STUB
                     org::kde::KDirNotify::emitFileRenamed(src, dst);
 #endif
@@ -251,7 +247,7 @@ void SimpleJob::slotWarning(const QString &errorText)
     Q_EMIT warning(this, errorText);
 }
 
-void SimpleJobPrivate::_k_slotSlaveInfoMessage(const QString &msg)
+void SimpleJobPrivate::_k_slotWorkerInfoMessage(const QString &msg)
 {
     Q_EMIT q_func()->infoMessage(q_func(), msg);
 }
@@ -285,9 +281,9 @@ void SimpleJobPrivate::slotSpeed(unsigned long speed)
 void SimpleJobPrivate::restartAfterRedirection(QUrl *redirectionUrl)
 {
     Q_Q(SimpleJob);
-    // Return slave to the scheduler while we still have the old URL in place; the scheduler
+    // Return worker to the scheduler while we still have the old URL in place; the scheduler
     // requires a job URL to stay invariant while the job is running.
-    slaveDone();
+    workerDone();
 
     m_url = *redirectionUrl;
     redirectionUrl->clear();
@@ -310,21 +306,16 @@ void SimpleJob::slotMetaData(const KIO::MetaData &_metaData)
     }
 
     // Update the internal meta-data values as soon as possible. Waiting until
-    // the ioslave is finished has unintended consequences if the client starts
-    // a new connection without waiting for the ioslave to finish.
+    // the KIO worker is finished has unintended consequences if the client starts
+    // a new connection without waiting for the KIO worker to finish.
     if (!d->m_internalMetaData.isEmpty()) {
         Scheduler::updateInternalMetaData(this);
     }
 }
 
-void SimpleJob::storeSSLSessionFromJob(const QUrl &redirectionURL)
-{
-    Q_UNUSED(redirectionURL);
-}
-
 void SimpleJobPrivate::slotPrivilegeOperationRequested()
 {
-    m_slave->send(MSG_PRIVILEGE_EXEC, privilegeOperationData());
+    m_worker->send(MSG_PRIVILEGE_EXEC, privilegeOperationData());
 }
 
 //////////
