@@ -840,7 +840,12 @@ void HTTPProtocol::davParsePropstats(const QDomNodeList &propstats, KIO::UDSEntr
 
             if (property.tagName() == QLatin1String("creationdate")) {
                 // Resource creation date. Should be is ISO 8601 format.
-                entry.replace(KIO::UDSEntry::UDS_CREATION_TIME, parseDateTime(property.text(), property.attribute(QStringLiteral("dt"))).toSecsSinceEpoch());
+                auto datetime = parseDateTime(property.text(), property.attribute(QStringLiteral("dt")));
+                if (datetime.isValid()) {
+                    entry.replace(KIO::UDSEntry::UDS_CREATION_TIME, datetime.toSecsSinceEpoch());
+                } else {
+                    qWarning() << "Failed to parse creationdate" << property.text() << property.attribute(QStringLiteral("dt"));
+                }
             } else if (property.tagName() == QLatin1String("getcontentlength")) {
                 // Content length (file size)
                 entry.replace(KIO::UDSEntry::UDS_SIZE, property.text().toULong());
@@ -875,8 +880,12 @@ void HTTPProtocol::davParsePropstats(const QDomNodeList &propstats, KIO::UDSEntr
 
             } else if (property.tagName() == QLatin1String("getlastmodified")) {
                 // Last modification date
-                entry.replace(KIO::UDSEntry::UDS_MODIFICATION_TIME,
-                              parseDateTime(property.text(), property.attribute(QStringLiteral("dt"))).toSecsSinceEpoch());
+                auto datetime = parseDateTime(property.text(), property.attribute(QStringLiteral("dt")));
+                if (datetime.isValid()) {
+                    entry.replace(KIO::UDSEntry::UDS_MODIFICATION_TIME, datetime.toSecsSinceEpoch());
+                } else {
+                    qWarning() << "Failed to parse getlastmodified" << property.text() << property.attribute(QStringLiteral("dt"));
+                }
             } else if (property.tagName() == QLatin1String("getetag")) {
                 // Entity tag
                 setMetaData(QStringLiteral("davEntityTag"), property.text());
@@ -992,12 +1001,18 @@ QDateTime HTTPProtocol::parseDateTime(const QString &input, const QString &type)
 {
     if (type == QLatin1String("dateTime.tz")) {
         return QDateTime::fromString(input, Qt::ISODate);
-    } else if (type == QLatin1String("dateTime.rfc1123")) {
-        return QDateTime::fromString(input, Qt::RFC2822Date);
+    }
+
+    // Qt decided to no longer support "GMT" for some reason: QTBUG-114681
+    QString inputUtc = input;
+    inputUtc.replace(QLatin1String("GMT"), QLatin1String("+0000"));
+
+    if (type == QLatin1String("dateTime.rfc1123")) {
+        return QDateTime::fromString(inputUtc, Qt::RFC2822Date);
     }
 
     // format not advertised... try to parse anyway
-    QDateTime time = QDateTime::fromString(input, Qt::RFC2822Date);
+    QDateTime time = QDateTime::fromString(inputUtc, Qt::RFC2822Date);
     if (time.isValid()) {
         return time;
     }
@@ -1186,7 +1201,7 @@ bool HTTPProtocol::davDestinationExists(const QUrl &url)
 KIO::WorkerResult HTTPProtocol::davGeneric(const QUrl &url, KIO::HTTP_METHOD method, qint64 size)
 {
     // TODO what about size?
-
+    Q_UNUSED(size)
     QMap<QByteArray, QByteArray> extraHeaders;
 
     if (method == KIO::DAV_PROPFIND || method == KIO::DAV_REPORT) {
@@ -1584,6 +1599,10 @@ KIO::WorkerResult HTTPProtocol::sendHttpError(const QUrl &url, KIO::HTTP_METHOD
     }
 
     if (errorCode) {
+        if (errorCode == KIO::ERR_DOES_NOT_EXIST) {
+            errorString = url.toDisplayString();
+        }
+
         return KIO::WorkerResult::fail(errorCode, errorString);
     }
 
@@ -1638,3 +1657,4 @@ QByteArray HTTPProtocol::methodToString(KIO::HTTP_METHOD method)
 }
 
 #include "http.moc"
+#include "moc_http.cpp"
