@@ -162,13 +162,39 @@ int KFileItemActionsPrivate::insertServicesSubmenus(const QMap<QString, ServiceL
 
 int KFileItemActionsPrivate::insertServices(const ServiceList &list, QMenu *menu)
 {
-    ServiceList sortedList = list;
-    std::sort(sortedList.begin(), sortedList.end(), [](const KDesktopFileAction &a1, const KDesktopFileAction &a2) {
-        return a1.name() < a2.name();
-    });
-    int count = 0;
-    for (const KDesktopFileAction &serviceAction : std::as_const(sortedList)) {
+    // Temporary storage for current group and all groups
+    ServiceList currentGroup;
+    std::vector<ServiceList> allGroups;
+
+    // Grouping
+    for (const KDesktopFileAction &serviceAction : std::as_const(list)) {
         if (serviceAction.isSeparator()) {
+            if (!currentGroup.empty()) {
+                allGroups.push_back(currentGroup);
+                currentGroup.clear();
+            }
+            // Push back a dummy list to represent a separator for later
+            allGroups.push_back(ServiceList());
+        } else {
+            currentGroup.push_back(serviceAction);
+        }
+    }
+    // Don't forget to add the last group if it exists
+    if (!currentGroup.empty()) {
+        allGroups.push_back(currentGroup);
+    }
+
+    // Sort each group
+    for (ServiceList &group : allGroups) {
+        std::sort(group.begin(), group.end(), [](const KDesktopFileAction &a1, const KDesktopFileAction &a2) {
+            return a1.name() < a2.name();
+        });
+    }
+
+    int count = 0;
+    for (const ServiceList &group : allGroups) {
+        // Check if the group is a separator
+        if (group.empty()) {
             const QList<QAction *> actions = menu->actions();
             if (!actions.isEmpty() && !actions.last()->isSeparator()) {
                 menu->addSeparator();
@@ -176,19 +202,22 @@ int KFileItemActionsPrivate::insertServices(const ServiceList &list, QMenu *menu
             continue;
         }
 
-        QAction *act = new QAction(q);
-        act->setObjectName(QStringLiteral("menuaction")); // for the unittest
-        QString text = serviceAction.name();
-        text.replace(QLatin1Char('&'), QLatin1String("&&"));
-        act->setText(text);
-        if (!serviceAction.icon().isEmpty()) {
-            act->setIcon(QIcon::fromTheme(serviceAction.icon()));
+        // Insert sorted actions for current group
+        for (const KDesktopFileAction &serviceAction : group) {
+            QAction *act = new QAction(q);
+            act->setObjectName(QStringLiteral("menuaction")); // for the unittest
+            QString text = serviceAction.name();
+            text.replace(QLatin1Char('&'), QLatin1String("&&"));
+            act->setText(text);
+            if (!serviceAction.icon().isEmpty()) {
+                act->setIcon(QIcon::fromTheme(serviceAction.icon()));
+            }
+            act->setData(QVariant::fromValue(serviceAction));
+            m_executeServiceActionGroup.addAction(act);
+            
+            menu->addAction(act); // Add to toplevel menu
+            ++count;
         }
-        act->setData(QVariant::fromValue(serviceAction));
-        m_executeServiceActionGroup.addAction(act);
-
-        menu->addAction(act); // Add to toplevel menu
-        ++count;
     }
 
     return count;
@@ -245,35 +274,32 @@ void KFileItemActions::addActionsTo(QMenu *menu, MenuActionSources sources, cons
 // static
 KService::List KFileItemActions::associatedApplications(const QStringList &mimeTypeList)
 {
-    return KFileItemActionsPrivate::associatedApplications(mimeTypeList, QString(), QStringList{});
+    return KFileItemActionsPrivate::associatedApplications(mimeTypeList, QStringList{});
 }
 
-static KService::Ptr preferredService(const QString &mimeType, const QStringList &excludedDesktopEntryNames, const QString &constraint)
+static KService::Ptr preferredService(const QString &mimeType, const QStringList &excludedDesktopEntryNames)
 {
-    KService::List services;
-    if (constraint.isEmpty()) {
-        services = KApplicationTrader::queryByMimeType(mimeType, [&](const KService::Ptr &serv) {
-            return !excludedDesktopEntryNames.contains(serv->desktopEntryName());
-        });
-    }
+    KService::List services = KApplicationTrader::queryByMimeType(mimeType, [&](const KService::Ptr &serv) {
+        return !excludedDesktopEntryNames.contains(serv->desktopEntryName());
+    });
     return services.isEmpty() ? KService::Ptr() : services.first();
 }
 
 void KFileItemActions::insertOpenWithActionsTo(QAction *before, QMenu *topMenu, const QStringList &excludedDesktopEntryNames)
 {
-    d->insertOpenWithActionsTo(before, topMenu, excludedDesktopEntryNames, QString());
+    d->insertOpenWithActionsTo(before, topMenu, excludedDesktopEntryNames);
 }
 
 void KFileItemActionsPrivate::slotRunPreferredApplications()
 {
     const KFileItemList fileItems = m_fileOpenList;
     const QStringList mimeTypeList = listMimeTypes(fileItems);
-    const QStringList serviceIdList = listPreferredServiceIds(mimeTypeList, QStringList(), m_traderConstraint);
+    const QStringList serviceIdList = listPreferredServiceIds(mimeTypeList, QStringList());
 
     for (const QString &serviceId : serviceIdList) {
         KFileItemList serviceItems;
         for (const KFileItem &item : fileItems) {
-            const KService::Ptr serv = preferredService(item.mimetype(), QStringList(), m_traderConstraint);
+            const KService::Ptr serv = preferredService(item.mimetype(), QStringList());
             const QString preferredServiceId = serv ? serv->storageId() : QString();
             if (preferredServiceId == serviceId) {
                 serviceItems << item;
@@ -296,7 +322,6 @@ void KFileItemActionsPrivate::slotRunPreferredApplications()
 void KFileItemActions::runPreferredApplications(const KFileItemList &fileOpenList)
 {
     d->m_fileOpenList = fileOpenList;
-    d->m_traderConstraint = QString();
     d->slotRunPreferredApplications();
 }
 
@@ -350,13 +375,12 @@ QStringList KFileItemActionsPrivate::listMimeTypes(const KFileItemList &items)
     return mimeTypeList;
 }
 
-QStringList
-KFileItemActionsPrivate::listPreferredServiceIds(const QStringList &mimeTypeList, const QStringList &excludedDesktopEntryNames, const QString &traderConstraint)
+QStringList KFileItemActionsPrivate::listPreferredServiceIds(const QStringList &mimeTypeList, const QStringList &excludedDesktopEntryNames)
 {
     QStringList serviceIdList;
     serviceIdList.reserve(mimeTypeList.size());
     for (const QString &mimeType : mimeTypeList) {
-        const KService::Ptr serv = preferredService(mimeType, excludedDesktopEntryNames, traderConstraint);
+        const KService::Ptr serv = preferredService(mimeType, excludedDesktopEntryNames);
         serviceIdList << (serv ? serv->storageId() : QString()); // empty string means mimetype has no associated apps
     }
     serviceIdList.removeDuplicates();
@@ -492,7 +516,7 @@ KFileItemActionsPrivate::addServiceActionsTo(QMenu *mainMenu, const QList<QActio
 
             ServiceList &list = s.selectList(priority, submenuName);
             std::copy_if(actions.cbegin(), actions.cend(), std::back_inserter(list), [&excludeList, &showGroup](const KDesktopFileAction &srvAction) {
-                return showGroup.readEntry(srvAction.name(), true) && !excludeList.contains(srvAction.name());
+                return showGroup.readEntry(srvAction.actionsKey(), true) && !excludeList.contains(srvAction.actionsKey());
             });
         }
     }
@@ -570,19 +594,15 @@ int KFileItemActionsPrivate::addPluginActionsTo(QMenu *mainMenu, QMenu *actionsM
     return itemCount;
 }
 
-KService::List
-KFileItemActionsPrivate::associatedApplications(const QStringList &mimeTypeList, const QString &traderConstraint, const QStringList &excludedDesktopEntryNames)
+KService::List KFileItemActionsPrivate::associatedApplications(const QStringList &mimeTypeList, const QStringList &excludedDesktopEntryNames)
 {
     if (!KAuthorized::authorizeAction(QStringLiteral("openwith")) || mimeTypeList.isEmpty()) {
         return KService::List();
     }
 
-    KService::List firstOffers;
-    if (traderConstraint.isEmpty()) {
-        firstOffers = KApplicationTrader::queryByMimeType(mimeTypeList.first(), [excludedDesktopEntryNames](const KService::Ptr &service) {
-            return !excludedDesktopEntryNames.contains(service->desktopEntryName());
-        });
-    }
+    KService::List firstOffers = KApplicationTrader::queryByMimeType(mimeTypeList.first(), [excludedDesktopEntryNames](const KService::Ptr &service) {
+        return !excludedDesktopEntryNames.contains(service->desktopEntryName());
+    });
 
     QList<KFileItemActionsPrivate::ServiceRank> rankings;
     QStringList serviceList;
@@ -602,13 +622,10 @@ KFileItemActionsPrivate::associatedApplications(const QStringList &mimeTypeList,
     }
 
     for (int j = 1; j < mimeTypeList.count(); ++j) {
-        KService::List offers;
         QStringList subservice; // list of services that support this MIME type
-        if (traderConstraint.isEmpty()) {
-            offers = KApplicationTrader::queryByMimeType(mimeTypeList[j], [excludedDesktopEntryNames](const KService::Ptr &service) {
-                return !excludedDesktopEntryNames.contains(service->desktopEntryName());
-            });
-        }
+        KService::List offers = KApplicationTrader::queryByMimeType(mimeTypeList[j], [excludedDesktopEntryNames](const KService::Ptr &service) {
+            return !excludedDesktopEntryNames.contains(service->desktopEntryName());
+        });
 
         subservice.reserve(offers.count());
         for (int i = 0; i != offers.count(); ++i) {
@@ -645,18 +662,14 @@ KFileItemActionsPrivate::associatedApplications(const QStringList &mimeTypeList,
     return result;
 }
 
-void KFileItemActionsPrivate::insertOpenWithActionsTo(QAction *before,
-                                                      QMenu *topMenu,
-                                                      const QStringList &excludedDesktopEntryNames,
-                                                      const QString &traderConstraint)
+void KFileItemActionsPrivate::insertOpenWithActionsTo(QAction *before, QMenu *topMenu, const QStringList &excludedDesktopEntryNames)
 {
     if (!KAuthorized::authorizeAction(QStringLiteral("openwith"))) {
         return;
     }
 
-    m_traderConstraint = traderConstraint;
     // TODO Overload with excludedDesktopEntryNames, but this method in public API and will be handled in a new MR
-    KService::List offers = associatedApplications(m_mimeTypeList, traderConstraint, excludedDesktopEntryNames);
+    KService::List offers = associatedApplications(m_mimeTypeList, excludedDesktopEntryNames);
 
     //// Ok, we have everything, now insert
 
@@ -701,7 +714,7 @@ void KFileItemActionsPrivate::insertOpenWithActionsTo(QAction *before,
     }
 #endif
 
-    QStringList serviceIdList = listPreferredServiceIds(m_mimeTypeList, excludedDesktopEntryNames, traderConstraint);
+    QStringList serviceIdList = listPreferredServiceIds(m_mimeTypeList, excludedDesktopEntryNames);
 
     // When selecting files with multiple MIME types, offer either "open with <app for all>"
     // or a generic <open> (if there are any apps associated).
@@ -710,7 +723,7 @@ void KFileItemActionsPrivate::insertOpenWithActionsTo(QAction *before,
 
         QAction *runAct = new QAction(this);
         if (serviceIdList.count() == 1) {
-            const KService::Ptr app = preferredService(m_mimeTypeList.first(), excludedDesktopEntryNames, traderConstraint);
+            const KService::Ptr app = preferredService(m_mimeTypeList.first(), excludedDesktopEntryNames);
             runAct->setText(isDir ? i18n("&Open folder with %1", app->name()) : i18n("&Open with %1", app->name()));
             runAct->setIcon(QIcon::fromTheme(app->icon()));
 
@@ -728,7 +741,6 @@ void KFileItemActionsPrivate::insertOpenWithActionsTo(QAction *before,
         QObject::connect(runAct, &QAction::triggered, this, &KFileItemActionsPrivate::slotRunPreferredApplications);
         topMenu->insertAction(before, runAct);
 
-        m_traderConstraint = traderConstraint;
         m_fileOpenList = m_props.items();
     }
 
