@@ -44,6 +44,7 @@
 #include <kurlauthorized.h>
 #include <kurlcompletion.h>
 #include <kurlrequester.h>
+#include <openwith.h>
 
 #include <KConfigGroup>
 #include <assert.h>
@@ -916,234 +917,27 @@ void KOpenWithDialog::setSaveNewApplications(bool b)
     d->saveNewApps = b;
 }
 
-static QString simplifiedExecLineFromService(const QString &fullExec)
-{
-    QString exec = fullExec;
-    exec.remove(QLatin1String("%u"), Qt::CaseInsensitive);
-    exec.remove(QLatin1String("%f"), Qt::CaseInsensitive);
-    exec.remove(QLatin1String("-caption %c"));
-    exec.remove(QLatin1String("-caption \"%c\""));
-    exec.remove(QLatin1String("%i"));
-    exec.remove(QLatin1String("%m"));
-    return exec.simplified();
-}
-
-void KOpenWithDialogPrivate::addToMimeAppsList(const QString &serviceId /*menu id or storage id*/)
-{
-    KSharedConfig::Ptr profile = KSharedConfig::openConfig(QStringLiteral("mimeapps.list"), KConfig::NoGlobals, QStandardPaths::GenericConfigLocation);
-
-    // Save the default application according to mime-apps-spec 1.0
-    KConfigGroup defaultApp(profile, QStringLiteral("Default Applications"));
-    defaultApp.writeXdgListEntry(qMimeType, QStringList(serviceId));
-
-    KConfigGroup addedApps(profile, QStringLiteral("Added Associations"));
-    QStringList apps = addedApps.readXdgListEntry(qMimeType);
-    apps.removeAll(serviceId);
-    apps.prepend(serviceId); // make it the preferred app
-    addedApps.writeXdgListEntry(qMimeType, apps);
-
-    profile->sync();
-
-    // Also make sure the "auto embed" setting for this MIME type is off
-    KSharedConfig::Ptr fileTypesConfig = KSharedConfig::openConfig(QStringLiteral("filetypesrc"), KConfig::NoGlobals);
-    fileTypesConfig->group(QStringLiteral("EmbedSettings")).writeEntry(QStringLiteral("embed-") + qMimeType, false);
-    fileTypesConfig->sync();
-
-    // qDebug() << "rebuilding ksycoca...";
-
-    // kbuildsycoca is the one reading mimeapps.list, so we need to run it now
-#ifndef KIO_ANDROID_STUB
-    KBuildSycocaProgressDialog::rebuildKSycoca(q);
-#endif
-
-    // could be nullptr if the user canceled the dialog...
-    m_pService = KService::serviceByStorageId(serviceId);
-}
-
 bool KOpenWithDialogPrivate::checkAccept()
 {
-    const QString typedExec(edit->text());
-    QString fullExec(typedExec);
-
-    QString serviceName;
-    QString initialServiceName;
-    QString preferredTerminal;
-    QString configPath;
-    QString serviceExec;
+    auto result = KIO::OpenWith::accept(curService,
+                                        edit->text(),
+                                        remember && remember->isChecked(),
+                                        qMimeType,
+                                        terminal->isChecked(),
+                                        nocloseonexit->isChecked(),
+                                        saveNewApps);
     m_pService = curService;
-    if (!m_pService) {
-        // No service selected - check the command line
 
-        // Find out the name of the service from the command line, removing args and paths
-        serviceName = KIO::DesktopExecParser::executableName(typedExec);
-        if (serviceName.isEmpty()) {
-            KMessageBox::error(q, i18n("Could not extract executable name from '%1', please type a valid program name.", serviceName));
-            return false;
-        }
-        initialServiceName = serviceName;
-        // Also remember the executableName with a path, if any, for the
-        // check that the executable exists.
-        // qDebug() << "initialServiceName=" << initialServiceName;
-        int i = 1; // We have app, app-2, app-3... Looks better for the user.
-        bool ok = false;
-        // Check if there's already a service by that name, with the same Exec line
-        do {
-            // qDebug() << "looking for service" << serviceName;
-            KService::Ptr serv = KService::serviceByDesktopName(serviceName);
-            ok = !serv; // ok if no such service yet
-            // also ok if we find the exact same service (well, "kwrite" == "kwrite %U")
-            if (serv && !serv->noDisplay() /* #297720 */) {
-                if (serv->isApplication()) {
-                    /*// qDebug() << "typedExec=" << typedExec
-                      << "serv->exec=" << serv->exec()
-                      << "simplifiedExecLineFromService=" << simplifiedExecLineFromService(fullExec);*/
-                    serviceExec = simplifiedExecLineFromService(serv->exec());
-                    if (typedExec == serviceExec) {
-                        ok = true;
-                        m_pService = serv;
-                        // qDebug() << "OK, found identical service: " << serv->entryPath();
-                    } else {
-                        // qDebug() << "Exec line differs, service says:" << serviceExec;
-                        configPath = serv->entryPath();
-                        serviceExec = serv->exec();
-                    }
-                } else {
-                    // qDebug() << "Found, but not an application:" << serv->entryPath();
-                }
-            }
-            if (!ok) { // service was found, but it was different -> keep looking
-                ++i;
-                serviceName = initialServiceName + QLatin1Char('-') + QString::number(i);
-            }
-        } while (!ok);
-    }
-    if (m_pService) {
-        // Existing service selected
-        serviceName = m_pService->name();
-        initialServiceName = serviceName;
-        fullExec = m_pService->exec();
-    } else {
-        const QString binaryName = KIO::DesktopExecParser::executablePath(typedExec);
-        // qDebug() << "binaryName=" << binaryName;
-        // Ensure that the typed binary name actually exists (#81190)
-        if (QStandardPaths::findExecutable(binaryName).isEmpty()) {
-            // QStandardPaths::findExecutable does not find non-executable files.
-            // Give a better error message for the case of a existing but non-executable file.
-            // https://bugs.kde.org/show_bug.cgi?id=437880
-            const QString msg = QFileInfo::exists(binaryName)
-                ? xi18nc("@info", "<filename>%1</filename> does not appear to be an executable program.", binaryName)
-                : xi18nc("@info", "<filename>%1</filename> was not found; please enter a valid path to an executable program.", binaryName);
-
-            KMessageBox::error(q, msg);
-            return false;
-        }
+    if (!result.accept) {
+        KMessageBox::error(q, result.error);
+        return false;
     }
 
-    if (terminal->isChecked()) {
-        KConfigGroup confGroup(KSharedConfig::openConfig(), QStringLiteral("General"));
-        preferredTerminal = confGroup.readPathEntry("TerminalApplication", QStringLiteral("konsole"));
-        m_command = preferredTerminal;
-        // only add --noclose when we are sure it is konsole we're using
-        if (preferredTerminal == QLatin1String("konsole") && nocloseonexit->isChecked()) {
-            m_command += QStringLiteral(" --noclose");
-        }
-        m_command += QLatin1String(" -e ") + edit->text();
-        // qDebug() << "Setting m_command to" << m_command;
+#ifndef KIO_ANDROID_STUB
+    if (result.rebuildSycoca) {
+        KBuildSycocaProgressDialog::rebuildKSycoca(q);
     }
-    if (m_pService && terminal->isChecked() != m_pService->terminal()) {
-        m_pService = nullptr; // It's not exactly this service we're running
-    }
-
-    const bool bRemember = remember && remember->isChecked();
-    // qDebug() << "bRemember=" << bRemember << "service found=" << m_pService;
-    if (m_pService) {
-        if (bRemember) {
-            // Associate this app with qMimeType in mimeapps.list
-            Q_ASSERT(!qMimeType.isEmpty()); // we don't show the remember checkbox otherwise
-            addToMimeAppsList(m_pService->storageId());
-        }
-    } else {
-        const bool createDesktopFile = bRemember || saveNewApps;
-        if (!createDesktopFile) {
-            // Create temp service
-            if (configPath.isEmpty()) {
-                m_pService = new KService(initialServiceName, fullExec, QString());
-            } else {
-                if (!typedExec.contains(QLatin1String("%u"), Qt::CaseInsensitive) && !typedExec.contains(QLatin1String("%f"), Qt::CaseInsensitive)) {
-                    int index = serviceExec.indexOf(QLatin1String("%u"), 0, Qt::CaseInsensitive);
-                    if (index == -1) {
-                        index = serviceExec.indexOf(QLatin1String("%f"), 0, Qt::CaseInsensitive);
-                    }
-                    if (index > -1) {
-                        fullExec += QLatin1Char(' ') + QStringView(serviceExec).mid(index, 2);
-                    }
-                }
-                // qDebug() << "Creating service with Exec=" << fullExec;
-                m_pService = new KService(configPath);
-                m_pService->setExec(fullExec);
-            }
-            if (terminal->isChecked()) {
-                m_pService->setTerminal(true);
-                // only add --noclose when we are sure it is konsole we're using
-                if (preferredTerminal == QLatin1String("konsole") && nocloseonexit->isChecked()) {
-                    m_pService->setTerminalOptions(QStringLiteral("--noclose"));
-                }
-            }
-        } else {
-            // If we got here, we can't seem to find a service for what they wanted. Create one.
-
-            QString menuId;
-#ifdef Q_OS_WIN32
-            // on windows, do not use the complete path, but only the default name.
-            serviceName = QFileInfo(serviceName).fileName();
 #endif
-            QString newPath = KService::newServicePath(false /* ignored argument */, serviceName, &menuId);
-            // qDebug() << "Creating new service" << serviceName << "(" << newPath << ")" << "menuId=" << menuId;
-
-            KDesktopFile desktopFile(newPath);
-            KConfigGroup cg = desktopFile.desktopGroup();
-            cg.writeEntry("Type", "Application");
-
-            // For the user visible name, use the executable name with any
-            // arguments appended, but with desktop-file specific expansion
-            // arguments removed. This is done to more clearly communicate the
-            // actual command used to the user and makes it easier to
-            // distinguish things like "qdbus".
-            QString name = KIO::DesktopExecParser::executableName(fullExec);
-            auto view = QStringView{fullExec}.trimmed();
-            int index = view.indexOf(QLatin1Char(' '));
-            if (index > 0) {
-                name.append(view.mid(index));
-            }
-            cg.writeEntry("Name", simplifiedExecLineFromService(name));
-
-            // if we select a binary for a scheme handler, then it's safe to assume it can handle URLs
-            if (qMimeType.startsWith(QLatin1String("x-scheme-handler/"))) {
-                if (!typedExec.contains(QLatin1String("%u"), Qt::CaseInsensitive) && !typedExec.contains(QLatin1String("%f"), Qt::CaseInsensitive)) {
-                    fullExec += QStringLiteral(" %u");
-                }
-            }
-
-            cg.writeEntry("Exec", fullExec);
-            cg.writeEntry("NoDisplay", true); // don't make it appear in the K menu
-            if (terminal->isChecked()) {
-                cg.writeEntry("Terminal", true);
-                // only add --noclose when we are sure it is konsole we're using
-                if (preferredTerminal == QLatin1String("konsole") && nocloseonexit->isChecked()) {
-                    cg.writeEntry("TerminalOptions", "--noclose");
-                }
-            }
-            if (!qMimeType.isEmpty()) {
-                cg.writeXdgListEntry("MimeType", QStringList() << qMimeType);
-            }
-            cg.sync();
-
-            if (!qMimeType.isEmpty()) {
-                addToMimeAppsList(menuId);
-            }
-            m_pService = new KService(newPath);
-        }
-    }
 
     saveComboboxHistory();
     return true;
